<aside class="sidebar">
    <div class="sidebar-header">
        <h2 id="logo-link" style="cursor: pointer;"><i class="fas fa-atom"></i> Deep Research</h2>
    </div>
    <nav class="sidebar-nav">
        <ul>
            <li {% if active_page == 'new-research' %}class="active"{% endif %} data-page="new-research">
                <a href="/"><i class="fas fa-search"></i> New Research</a>
            </li>
            <li {% if active_page == 'history' %}class="active"{% endif %} data-page="history">
                <a href="{{ url_for('history.history_page') }}"><i class="fas fa-history"></i> History</a>
            </li>
            <li {% if active_page == 'news' %}class="active"{% endif %} data-page="news">
                <a href="{{ url_for('news.news_page') }}"><i class="fas fa-newspaper"></i> News</a>
            </li>
            <li {% if active_page == 'subscriptions' %}class="active"{% endif %} data-page="subscriptions">
                <a href="{{ url_for('news.subscriptions_page') }}"><i class="fas fa-bell"></i> Subscriptions</a>
            </li>
            <li {% if active_page == 'benchmark' %}class="active"{% endif %} data-page="benchmark">
                <a href="{{ url_for('benchmark.index') }}"><i class="fas fa-tachometer-alt"></i> Benchmark</a>
            </li>
            <li {% if active_page == 'benchmark-results' %}class="active"{% endif %} data-page="benchmark-results">
                <a href="{{ url_for('benchmark.results') }}"><i class="fas fa-chart-line"></i> Benchmark Results</a>
            </li>
            <li {% if active_page == 'metrics' %}class="active"{% endif %} data-page="metrics">
                <a href="{{ url_for('metrics.metrics_dashboard') }}"><i class="fas fa-chart-bar"></i> Metrics</a>
            </li>
            <li {% if active_page == 'settings' %}class="active"{% endif %} data-page="settings">
                <a href="{{ url_for('settings.settings_page') }}"><i class="fas fa-cog"></i> Settings</a>
            </li>
        </ul>
    </nav>
    <div class="sidebar-footer">
        <p>{{ version }} | <i class="fas fa-brain"></i></p>
    </div>
</aside>
