# config/enums.py
from enum import Enum


class KnowledgeAccumulationApproach(Enum):
    QUESTION = "QUESTION"
    ITERATION = "ITERATION"
    NO_KNOWLEDGE = "NO_KNOWLEDGE"
    MAX_NR_OF_CHARACTERS = "MAX_NR_OF_CHARACTERS"
